/**
 * gclc-process:net.bigeon.gclc.process.CommandFork.java
 * Created on: Nov 13, 2017
 */
package net.bigeon.gclc.process;

import java.text.MessageFormat;
/*-
 * #%L
 * process
 * %%
 * Copyright (C) 2014 - 2018 Bigeon
 * %%
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 * #L%
 */
import java.util.Arrays;

import net.bigeon.gclc.command.Command;
import net.bigeon.gclc.command.ICommand;
import net.bigeon.gclc.command.ICommandProvider;
import net.bigeon.gclc.exception.CommandRunException;
import net.bigeon.gclc.exception.CommandRunExceptionType;
import net.bigeon.gclc.manager.ConsoleInput;
import net.bigeon.gclc.manager.ConsoleOutput;

/** A command that is launched inside an internal terminal.
 * <p>
 * Several things are to be considered before adding this command to an
 * application:
 * <ul>
 * <li>The commands will be able to run in parallel.
 * <li>The managing of the commands will be handled through piped systems and
 * buffered in so switching to a forked command will actually print again all
 * the command history to the output console.
 * </ul>
 *
 * @author Emmanuel Bigeon */
public class CommandFork extends Command {

    /** The task pool containing the tasks. */
    private final TaskPool         pool;
    /** The command provider. */
    private final ICommandProvider provider;
    /** The number of lines stored in the commands. */
    private final int              lines;

    /** Add the fork command.
     *
     * @param name the command name
     * @param provider the allowed command collection */
    public CommandFork(final String name, final ICommandProvider provider,
                       final TaskPool pool) {
        this(name, provider, pool, -1);
    }

    /** Add the fork command.
     *
     * @param name the command name
     * @param provider the allowed command collection */
    public CommandFork(final String name, final ICommandProvider provider,
                       final TaskPool pool, final int lines) {
        super(name);
        this.provider = provider;
        this.pool = pool;
        this.lines = lines;
    }

    /* (non-Javadoc)
     * @see net.bigeon.gclc.command.ICommand#execute(net.bigeon.gclc.manager.
     * ConsoleOutput, net.bigeon.gclc.manager.ConsoleInput, java.lang.String[]) */
    @Override
    public void execute(final ConsoleOutput out, final ConsoleInput in,
            final String... args) throws CommandRunException {
        if (args.length < 1) {
            throw new CommandRunException(CommandRunExceptionType.USAGE,
                    "No command to fork");
        }
        final String string = args[0];
        final ICommand cmd = provider.get(string);
        if (cmd == null) {
            throw new CommandRunException(CommandRunExceptionType.USAGE,
                    MessageFormat.format("No such command {0}", string));
        }
        final String[] inner = Arrays.copyOfRange(args, 1, args.length);
        final ForkTask task = new ForkCommandTask(cmd, inner, lines);
        final Thread th = new Thread(task, MessageFormat.format("fork [{0}]", string));
        pool.add(task);
        th.start();
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.command.ICommand#tip() */
    @Override
    public String tip() {
        return "Command background launch";
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.command.Command#usageDetail() */
    @Override
    protected String usageDetail() {
        return "";
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.command.Command#usagePattern() */
    @Override
    protected String usagePattern() {
        return super.usagePattern() + " <command>";
    }
}
